#include "mbed.h"

//Definimos que el puerto serie se llama pc
Serial pc(USBTX, USBRX);

//Creamos los tres leds pra usar
DigitalOut led_red(LED1);
DigitalOut led_green(LED2);
DigitalOut led_blue(LED3);

//esta funcion la vamos a llamar con cada uno de los bytes recibidos
void protocol_process_data_in (unsigned char );
//se llama todo el tiempo para procesar el protocolo
void protocol_process (void);

//Callback cuando se detecta una entrada
void onCharReceived(){
    //Con el dato recibido llamamos a procesar el protcolo
    while(pc.readable())
        protocol_process_data_in ( pc.getc());
}

int main(){
    //Apagamos los leds
    led_red = 1;
    led_green = 1;
    led_blue = 1;

    //Ejecutar onCharReceived por cada entrada por puerto
    pc.attach(&onCharReceived);

    pc.printf("Ejemplo de protocolo con maquinas de estados\r\n");
    
    while (true)
    {
        protocol_process();
    }
}


/* Maquina estados para recepcion de protoclo */
/*
 PROTOCOLO DEFINIDO
    [:][LED][VALOR][XOR][$] 
*/

#define PROTOCOL_INIT ':'
#define PROTOCOL_LED_ROJO 'R'
#define PROTOCOL_LED_BLUE 'B'
#define PROTOCOL_LED_GREEN 'G'
#define PROTOCOL_LED_ON '1'
#define PROTOCOL_LED_OFF '0'
#define PROTOCOL_END '$'


//Definidmos los estados de la recepcion
enum{
    RX_PROTOCOL_INIT,
    RX_PROTOCOL_LED_INDEX,
    RX_PROTOCOL_LED_VALUE,
    RX_PROTOCOL_XOR,
    RX_PROTOCOL_END,
};

//en estas variables vamos almancenando los datos del protocolo
unsigned char protocol_led_index, protocol_led_value, protocol_xor_calculated, protocol_xor_rx;

//variable que nos indica si estamos listos para procesar
bool protocol_ready_to_proc = false;

void protocol_process (void){
    //hay datos para procesar?
    if( protocol_ready_to_proc == false)
        return ;
        
    //si fue true hacemos false y procesamos
    protocol_ready_to_proc = false;
    
    //si los xor dan mal devolvemos error
    if( protocol_xor_rx != protocol_xor_calculated ){
        pc.printf(":RX_ERR$");
        return;
    }
   
    //para operar una sola vez, nos guardamos la accion sobre el led 
    unsigned char value;
    
    if( protocol_led_value == PROTOCOL_LED_ON )
        value = 0;  //0 para encencer el led
    else if( protocol_led_value == PROTOCOL_LED_OFF )
        value = 1;  //1 para apagar el led
    else {
        //error en el protcolo
        pc.printf(":RX_ERR$");
        //nos vamos sin hacer nada sobre los leds
        return;
    }
    
    switch ( protocol_led_index ){
        case PROTOCOL_LED_ROJO:
            led_red = value;
            break;
            
        case PROTOCOL_LED_GREEN:
            led_green = value;
            break;
            
        case PROTOCOL_LED_BLUE:
            led_blue = value;
            break;
            
        default:
            //me pidieron un led que no corresponde
            pc.printf(":RX_ERR$");
            //nos vamos sin hacer nada sobre los leds
            return;
    }
    
    //si estamos acá respondemos OK el protocolo
    pc.printf("%c%c%c%c%c", PROTOCOL_INIT,
                            protocol_led_index,
                            protocol_led_value,
                            protocol_xor_rx,
                            PROTOCOL_END);
}

void protocol_process_data_in (unsigned char data){
    static unsigned char estado_recepcion = RX_PROTOCOL_INIT;
    
    switch( estado_recepcion ){
        case RX_PROTOCOL_INIT:
            protocol_xor_calculated = 0; //aca vamos a ir calculando el xor de lo que recibimos
                                         //siempre que estamos en el estado inicial la hacemos 0
            if( data == PROTOCOL_INIT ){
                estado_recepcion = RX_PROTOCOL_LED_INDEX;
            }else{
                //si lo que recibimos en el estado de INIT no es el INIT nos quedamos
                estado_recepcion = RX_PROTOCOL_INIT;
            }
            break;
        
        case RX_PROTOCOL_LED_INDEX:
            //guardamos el led que debemos operar
            protocol_led_index = data;
            //calculamos la xor del dato
            protocol_xor_calculated ^= data;
            estado_recepcion = RX_PROTOCOL_LED_VALUE;
            break;
        
        case RX_PROTOCOL_LED_VALUE:
            //guardamos el valor que debemos aplicar
            protocol_led_value = data;
            //calculamos la xor del dato
            protocol_xor_calculated ^= data;
            estado_recepcion = RX_PROTOCOL_XOR;
            break;
        
        case RX_PROTOCOL_XOR:
            protocol_xor_rx = data;
            estado_recepcion = RX_PROTOCOL_END;
            break;
        
        case RX_PROTOCOL_END:
            //llego el caracter de fin?
            if( data == PROTOCOL_END )
                protocol_ready_to_proc = true;
                
            estado_recepcion = RX_PROTOCOL_INIT;
            break;
    }
}